/*
 * grunt-imgs-sprite
 * https://github.com/Administrator/my-group-plugin
 *
 * Copyright (c) 2016 cxj
 * Licensed under the MIT license.
 */

'use strict';
var fs = require("fs");
var path = require('path');
var Spritesmith = require('spritesmith');
var EventProxy = require('eventproxy');
var async =require("async");

module.exports = function(grunt) {

  // Please see the Grunt documentation for more information regarding task
  // creation: http://gruntjs.com/creating-tasks

  grunt.registerMultiTask('imgs_sprite', 'imgs sprite', function() {
      var done = this.async();
    // Merge task-specific and/or target-specific options with these defaults.
    var options = this.options();
      var cssFiles = this.files;
      var spritePath = options.spritepath?options.spritepath:"../images"; // 替换后的背景路径
      if(spritePath.lastIndexOf('/')==spritePath.length-1){
          spritePath = spritePath.substr(0,spritePath.length-1);
      }

      var spritedest = options.spritedest?options.spritedest:"images"; //雪碧图输出目录
      if(spritedest.lastIndexOf('/')==spritedest.length-1){
          spritedest = spritedest.substr(0,spritedest.length-1);
      }

      var sourceImgPath = options.imagepath?options.imagepath:"../images/sprite"; //背景图源文件
      if(sourceImgPath.lastIndexOf('/')==sourceImgPath.length-1){
          sourceImgPath = sourceImgPath.substr(0,sourceImgPath.length-1);
      }

      var exts = '.png,.jpg'; // 允许合并的图片格式
      var file_exts = ".css";
      //var cssDestPath = "test/build/css"; //导出的css路径
      var startPathIndex = sourceImgPath.split('/').length;

      var spritesArray = []; //要合并的小图二维数组
      var spritesPaths = []; //要合并的图片路径，用于命名生成图片
      var imgSetMap = {}; //小图最终结果对应位置map

      //var testCssSrc = "test/css";

      /**
       * 创建多层文件夹 同步方式
       */
      function mkdirSync(dirpath){
          var newPath = path.resolve(dirpath);
          if(!fs.existsSync(newPath)){
              var pathTmp;
              newPath.split(path.sep).forEach(function(dirname){
                  if(pathTmp){
                        pathTmp = path.join(pathTmp,dirname);
                  }else{
                        pathTmp = dirname;
                  }

                  if(!fs.existsSync(pathTmp)){
                      fs.mkdirSync(pathTmp);
                  }

              });
          }
      }

      /**
       * 读取文件信息
       */
      function readDirInfo(dirPath){
          var files = fs.readdirSync(dirPath);
          var sprites = [];
          files.forEach(function(fileName){
              var filePath = dirPath+"/"+fileName;
              var stat = fs.statSync(filePath);
              if(!stat.isDirectory()){
                  var extName = path.extname(filePath);
                  if(exts.indexOf(extName) >= 0&&extName!=""){
                      // 图片
                      if(sprites.length==0){
                          spritesPaths.push(dirPath);
                      }
                      sprites.push(path.resolve(filePath));
                  }

              }else{
                  readDirInfo(filePath);
              }
          });
          if(sprites.length){
              spritesArray.push(sprites);
          }

      }

      // 读取文件信息
      readDirInfo(sourceImgPath);

      function fixPath(path) {
          return String(path).replace(/\\/g, '/').replace(/\/$/, '');
      }

      function getSliceData(src,options){
          var cssPath = path.resolve(path.dirname(src));
          var cssData = fs.readFileSync(src, {}).toString();

          var rurlParams = /\?.*$/;
          var rabsUrl = /^(?:\/|https?:|file:)/i;
          var rbgs = /\bbackground(?:-image)?\s*:[^;]*?url\((["\']?)([^\)]+)\1\)[^};]*;?/ig;


          cssData = cssData.replace(rbgs, function(css, b, uri) {
              var imgUri = uri;
              var imgFullPath = imgUri.replace(rurlParams, '');
              imgFullPath = fixPath(path.join(cssPath, imgFullPath));
              imgFullPath = path.normalize(imgFullPath);
              var cssItem = imgSetMap[imgFullPath];
             css.replace(uri,spritePath+"/"+cssItem.img);
              if(css.indexOf(";")==-1){
                  css+=";"
              }
              css+="background-position:-"+cssItem.x+"px -"+cssItem.y+"px;";
            return css;

          });

          return cssData;
          // 判断输出文件夹是否存在，不存在则创建
          /*    mkdirSync(cssDestPath);
              var cssName = path.basename(src);
              var outPath = path.join(cssDestPath,cssName);
              fs.writeFileSync(outPath,cssData);*/
      }

      /**
       * 重新生成雪碧图css
       */
      function reBuildSpriteCss(cssItem){
          var cssData = "";
          cssItem.src.forEach(function(css){
                cssData += getSliceData(css);
              grunt.log.writeln("!Done "+css);
          });
          var dirname = path.dirname(cssItem.dest);
          mkdirSync(dirname);
          fs.writeFileSync(cssItem.dest,cssData);
          grunt.log.writeln("create "+cssItem.dest);
          grunt.log.writeln();
      }

      /**
       * 雪碧图创建
       */
      function spriteBuild(list){
          list.forEach(function(data,index){
              var pathName = spritesPaths[list.length-index-1];
              var paths = pathName.split('/');
              var newImgName = "";
              if(startPathIndex==paths.length){
                  newImgName = paths[paths.length-1];
              }else{
                  for(var i=startPathIndex;i<paths.length;i++){
                      if(newImgName!=""){
                          newImgName+="_";
                      }
                      newImgName +=paths[i]
                  }
              }
              var imgItems = data.coordinates;
              for(var key in imgItems){
                  imgItems[key].img = newImgName+".png";
                  imgSetMap[key] = imgItems[key];
              }

              var newFilePath = spritedest + '/'+newImgName+'.png'
              // Output the image
              fs.writeFileSync(newFilePath, data.image);
              data.coordinates, data.properties; // Coordinates and properties

          });

         // reBuildSpriteCss(testCssSrc);
          cssFiles.forEach(function(cssItem){
                reBuildSpriteCss(cssItem);
          });
          done();

      }

      var ep = new EventProxy();
      ep.after("sprites",spritesArray.length,spriteBuild);

      /**
       * 循环雪碧图数组
       */
     spritesArray.forEach(function(sprites){
         Spritesmith.run({
             src: sprites,
             padding: 2 // Exaggerated for visibility, normally 1 or 2
         }, function handleResult (err, result) {
             // If there was an error, throw it
             if (err) {
                 throw err;
             }

             // 判断输出文件夹是否存在，不存在则创建
             mkdirSync(spritedest);

            ep.emit("sprites",result)

         });
     });

  });

};
